#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-11-09
#
# Monitor Check Point Maestro SMO interfaces
#
# 2021-03-03: rewrite for CMK 2.0
# 2021-09-10: changed render function to networkbandwidth
# 2021-09-11: added WTO options
# 2022-10-23: fixed warning on upgrade "non-empty params vanished" for asgNetIfSpeed
#

# sample snmpwalk (for one interface)
# .1.3.6.1.4.1.2620.1.48.26.1.1.1.4.0 = Gauge32: 4
# .1.3.6.1.4.1.2620.1.48.26.1.1.2.4.0 = STRING: "bond2.602"
# .1.3.6.1.4.1.2620.1.48.26.1.1.3.4.0 = STRING: "10.9.9.1/25"
# .1.3.6.1.4.1.2620.1.48.26.1.1.4.4.0 = STRING: "-"
# .1.3.6.1.4.1.2620.1.48.26.1.1.5.4.0 = STRING: "00:1c:7f:81:05:09"
# .1.3.6.1.4.1.2620.1.48.26.1.1.6.4.0 = STRING: "Vlan"
# .1.3.6.1.4.1.2620.1.48.26.1.1.7.4.0 = STRING: "(up)/(up)"
# .1.3.6.1.4.1.2620.1.48.26.1.1.8.4.0 = STRING: "NA"
# .1.3.6.1.4.1.2620.1.48.26.1.1.9.4.0 = STRING: "1500"
# .1.3.6.1.4.1.2620.1.48.26.1.1.10.4.0 = STRING: "Full"
# .1.3.6.1.4.1.2620.1.48.26.1.1.11.4.0 = STRING: "355320533"
# .1.3.6.1.4.1.2620.1.48.26.1.1.12.4.0 = STRING: "4673143507"
#
# sample section
# [
#  [u'bond2', u'-', u'-', u'00:1c:7f:81:05:09', u'Bond Master', u'(up)/(up)', u'20G', u'NA', u'Full', u'3373103274', u'5007776301'],
#  [u'eth1-05', u'-', u'-', u'00:1c:7f:81:05:09', u'Bond slave', u'(up)/(up)', u'10G', u'1500', u'Full', u'1014183605', u'3291515315'],
#  [u'eth2-05', u'-', u'-', u'00:1c:7f:81:05:09', u'Bond slave', u'(up)/(up)', u'10G', u'1500', u'Full', u'2358951560', u'1716293763'],
#  [u'bond2.602', u'10.9.9.1/25', u'-', u'00:1c:7f:81:05:09', u'Vlan', u'(up)/(up)', u'NA', u'1500', u'Full', u'355320533', u'4673143507'],
#  [u'bond2.3001', u'10.0.100.49/27', u'-', u'00:1c:7f:81:05:09', u'Vlan', u'(up)/(up)', u'NA', u'1500', u'Full', u'2910413426', u'229216384'],
#  [u'bond3', u'-', u'-', u'00:1c:7f:81:06:09', u'Bond Master', u'(up)/(up)', u'20G', u'NA', u'Full', u'1479928728', u'133617791'],
#  [u'eth1-06', u'-', u'-', u'00:1c:7f:81:06:09', u'Bond slave', u'(up)/(up)', u'10G', u'1500', u'Full', u'214459394', u'131774843'],
#  [u'eth2-06', u'-', u'-', u'00:1c:7f:81:06:09', u'Bond slave', u'(up)/(up)', u'10G', u'1500', u'Full', u'1265469477', u'1843158'],
#  [u'magg1', u'10.0.100.9/27', u'-', u'00:1c:7f:6e:9b:dc', u'Bond Master', u'(up)/(up)', u'20G', u'NA', u'Full', u'200631796', u'384654797'],
#  [u'eth1-Mgmt1', u'-', u'-', u'00:1c:7f:6e:9b:dc', u'Bond slave', u'(Up)/(Up)', u'10G', u'1500', u'Full', u'61520991', u'44776128'],
#  [u'eth2-Mgmt1', u'-', u'-', u'00:1c:7f:6e:9b:dc', u'Bond slave', u'(Up)/(Up)', u'10G', u'1500', u'Full', u'139110651', u'339878254'],
#  [u'Sync', u'192.0.2.1/24', u'-', u'00:1c:7f:01:04:fe', u'Bond Master', u'(up)/(up)', u'20G', u'NA', u'Full', u'24549684703', u'14214325659'],
#  [u'eth1-Sync', u'-', u'-', u'00:1c:7f:01:04:fe', u'Bond slave', u'(down)/(down)', u'10G', u'1500', u'Full', u'1228171', u'1511513'],
#  [u'eth2-Sync', u'-', u'-', u'00:1c:7f:01:04:fe', u'Bond slave', u'(up)/(up)', u'10G', u'1500', u'Full', u'24548432877', u'14212801896']
# ]
#

import time

from typing import Dict, List, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    StringTable,
    CheckResult,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    equals,
    Result,
    check_levels,
    State,
    SNMPTree,
    startswith,
    all_of,
    any_of,

    GetRateError,
    get_rate,
    get_value_store,
    IgnoreResults,
    render,
)


class CheckPointASGInterface(NamedTuple):
    asgNetIfIPv4Addr: str
    asgNetIfIPv6Addr: str
    asgNetIfMACAddr: str
    asgNetIfInfo: str
    asgNetIfState: str
    asgNetIfSpeed: str
    asgNetIfMTU: str
    asgNetIfDuplex: str
    asgNetIfRx: int
    asgNetIfTx: int


def parse_checkpoint_asg_interface(string_table: List[StringTable]) -> Dict[str, CheckPointASGInterface]:
    parsed = {}
    for entry in string_table[0]:
        asgNetIfName, asgNetIfIPv4Addr, asgNetIfIPv6Addr, asgNetIfMACAddr, asgNetIfInfo, asgNetIfState, \
        asgNetIfSpeed, asgNetIfMTU, asgNetIfDuplex, asgNetIfRx, asgNetIfTx = entry
        # if not asgNetIfIPv4Addr == asgNetIfIPv6Addr == '-':  # do not add interfaces without ip address
        parsed[asgNetIfName] = CheckPointASGInterface(
            asgNetIfIPv4Addr,
            asgNetIfIPv6Addr,
            asgNetIfMACAddr,
            asgNetIfInfo,
            asgNetIfState,
            asgNetIfSpeed,
            asgNetIfMTU,
            asgNetIfDuplex,
            int(asgNetIfRx),
            int(asgNetIfTx)
        )

    return parsed


def discovery_checkpoint_asg_interface(section: Dict[str, CheckPointASGInterface]) -> DiscoveryResult:
    for item in section.keys():
        yield Service(item=item, parameters={'asgNetIfSpeed': section[item].asgNetIfSpeed})


def check_checkpoint_asg_interface(item, params, section: Dict[str, CheckPointASGInterface]) -> CheckResult:
    try:
        asg_interface = section[item]
    except KeyError:
        yield Result(state=State.UNKNOWN, notice='Item not found in SNMP data.')
        return

    inv_asgNetIfSpeed = params['asgNetIfSpeed']

    now = time.time()
    value_store = get_value_store()

    for label, metric, value in [
        ('In', 'if_in_octets', asg_interface.asgNetIfRx),
        ('Out', 'if_out_octets', asg_interface.asgNetIfTx),
    ]:
        try:
            value = get_rate(value_store, label, now, value, raise_overflow=True, )
        except GetRateError as exc:
            yield IgnoreResults(str(exc))
        else:
            yield from check_levels(value, metric_name=metric, render_func=render.networkbandwidth, label=label, )

    details = ''
    details += f'\nIPv4 Address: {asg_interface.asgNetIfIPv4Addr}'
    details += f'\nIPv6 Address: {asg_interface.asgNetIfIPv6Addr}'
    details += f'\nMAC Address: {asg_interface.asgNetIfMACAddr}'
    details += f'\nType: {asg_interface.asgNetIfInfo}'
    details += f'\nMTU: {asg_interface.asgNetIfMTU}'
    details += f'\nDuplex: {asg_interface.asgNetIfDuplex}'

    if not asg_interface.asgNetIfIPv4Addr == '-':
        summary = f'IPv4: {asg_interface.asgNetIfIPv4Addr}'
    else:
        summary = 'IPv4: N/A'

    yield Result(state=State.OK, summary=summary, details=details)

    yield_text = f'State: {asg_interface.asgNetIfState}'
    if not asg_interface.asgNetIfState.lower() == '(up)/(up)':
        yield Result(state=State(params['state_not_up_up']), summary=yield_text)
    else:
        yield Result(state=State.OK, summary=yield_text)

    if not asg_interface.asgNetIfSpeed == inv_asgNetIfSpeed:
        yield Result(
            state=State(params['state_speed_changed']),
            summary=f'Speed: {asg_interface.asgNetIfSpeed} ({inv_asgNetIfSpeed} expected)'
        )
    elif not asg_interface.asgNetIfSpeed == 'NA':
        yield Result(state=State.OK, summary=f'Speed {asg_interface.asgNetIfSpeed}')

    if not asg_interface.asgNetIfDuplex.lower() == 'full':
        yield Result(
            state=State(params['state_not_full_duplex']),
            summary=f'Duplex: {asg_interface.asgNetIfDuplex} (Full expected)'
        )
    else:
        yield Result(state=State.OK, summary=f'Duplex: {asg_interface.asgNetIfDuplex}')


register.snmp_section(
    name='checkpoint_asg_interface',
    parse_function=parse_checkpoint_asg_interface,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.48.26.1.1',  # CHECKPOINT-MIB::asgResourceEntry
            oids=[
                '2',  # asgNetIfName
                '3',  # asgNetIfIPv4Addr
                '4',  # asgNetIfIPv6Addr
                '5',  # asgNetIfMACAddr
                '6',  # asgNetIfInfo
                '7',  # asgNetIfState
                '8',  # asgNetIfSpeed
                '9',  # asgNetIfMTU
                '10',  # asgNetIfDuplex
                '11',  # asgNetIfRx
                '12',  # asgNetIfTx
            ]
        ),
    ],
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_asg_interface',
    service_name='ASG Interface %s',
    discovery_function=discovery_checkpoint_asg_interface,
    check_function=check_checkpoint_asg_interface,
    check_default_parameters={
        'state_not_full_duplex': 2,
        'state_not_up_up': 2,
        'state_speed_changed': 1,
    },
    check_ruleset_name='checkpoint_asg_interface',
)
